import 'dart:convert';
import 'dart:developer';
import 'package:idkit/idkit.dart';

class IDKitLog with ILog, CLog, TLog {
  /// Log output options object.
  late LogOption optionLog;

  /// Log Management Simple Interest.
  static IDKitLog? _instance;
  static IDKitLog get instance => _instance ??= IDKitLog._init();

  /// Factory mode creates log management objects.
  factory IDKitLog.init({LogOption? option}) =>
      _instance ??= IDKitLog._init(option: option);

  /// Log management object core initialization method.
  IDKitLog._init({LogOption? option}) {
    optionLog = const LogOption().copy(option);
  }

  /// Log output for a single object.
  static value(Object? value, {String? mark, LogOption? option}) {
    final LogOption vOption = IDKitLog.instance.optionLog.copy(option);
    if (vOption.isEnable ?? true && value.isNoNull) {
      final StackTrace stackTrace = StackTrace.current;
      TLog.outPutModeMethod(vOption, stackTrace, LogMethod.value,
          value: value, mark: mark);
    }
  }

  /// Output of log collection type.
  static list(List<dynamic>? list,
      {String? mark, bool isExpand = false, LogOption? option}) {
    if (IDKitLog.instance.optionLog.isEnable ?? true && value.isNoNull) {
      final LogOption vOption = IDKitLog.instance.optionLog.copy(option);
      final StackTrace stackTrace = StackTrace.current;
      TLog.outPutModeMethod(vOption, stackTrace, LogMethod.list,
          mark: mark, list: list, isExpand: isExpand);
    }
  }

  /// Collection of values or elements.
  static iterable(Iterable<dynamic>? iterable,
      {String? mark, bool isExpand = false, LogOption? option}) {
    if (IDKitLog.instance.optionLog.isEnable ?? true && value.isNoNull) {
      final LogOption vOption = IDKitLog.instance.optionLog.copy(option);
      final StackTrace stackTrace = StackTrace.current;
      TLog.outPutModeMethod(vOption, stackTrace, LogMethod.iterable,
          mark: mark, iterable: iterable, isExpand: isExpand);
    }
  }

  /// The output of the log key-value pair collection type.
  static map(Map<dynamic, dynamic>? map,
      {String? mark, bool isExpand = false, LogOption? option}) {
    if (IDKitLog.instance.optionLog.isEnable ?? true && map.isNoNull) {
      final LogOption vOption = IDKitLog.instance.optionLog.copy(option);
      final StackTrace stackTrace = StackTrace.current;
      TLog.outPutModeMethod(vOption, stackTrace, LogMethod.map,
          mark: mark, map: map, isExpand: isExpand);
    }
  }

  /// Log output for a single object.
  static json(Object? object, {String? mark, LogOption? option}) {
    final LogOption vOption = IDKitLog.instance.optionLog.copy(option);
    if (vOption.isEnable ?? true && value.isNoNull) {
      final String content = jsonEncode(
        object,
        toEncodable: (nonEncodable) => nonEncodable.toString(),
      );
      final StackTrace stackTrace = StackTrace.current;
      TLog.outPutModeMethod(vOption, stackTrace, LogMethod.json,
          mark: mark, json: content);
    }
  }
}

mixin ILog {
  /// Get the current call log widget name.
  static String getWidgetName(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int widgetStart = outPut.indexOf(RegExp(r'[A-Za-z]'), 1);
    final int widgetEnd = outPut.indexOf('.', widgetStart);
    return outPut.substring(widgetStart, widgetEnd);
  }

  /// Method to get the current call log.
  static String getCallMethod(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int methodStart = outPut.indexOf('.', 1);
    final int methodEnd = outPut.indexOf(' (', methodStart);
    String method = outPut.substring(methodStart + 1, methodEnd);
    if (method.contains('.<')) {
      method = method.split('.<').first;
    }
    return method;
  }

  /// Get the file path of the current call log.
  static String getFilePath(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int filePathStart = outPut.indexOf('(', 1);
    final int filePathEnd = outPut.indexOf(RegExp(r':[0-9]'), filePathStart);
    return outPut.substring(filePathStart + 1, filePathEnd);
  }

  /// Get the number of lines in the current call log.
  static String getLine(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int lineStart = outPut.indexOf(RegExp(r'[0-9]+:'), 1);
    final int lineEnd = outPut.indexOf(RegExp(r':[0-9]'), lineStart);
    return outPut.substring(lineStart, lineEnd);
  }
}

/// Copywriting
mixin TLog {
  /// Head  and Footer of log
  static String getHeaderAndFooter(LogOption option, {bool type = true}) {
    final String content = type
        ? (option.language == Language.en ? 'LOG OUTPUT START' : '日志输出开始')
        : (option.language == Language.en ? ' LOG OUTPUT END ' : '日志输出结束');
    final String header =
        '${option.decorator! * 24} $content ${option.decorator! * 24}';
    return CLog.coloringMethod(option.colorTheme!, header);
  }

  /// Log output time.
  static String getTime(LogOption option) {
    final DateTime dateTime = DateTime.now();
    final String content = option.language == Language.en
        ? '   TIME: ${dateTime.format(style: 'hh:mm:ss MM/dd/yyyy')}'
        : '时间: ${dateTime.format()}';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// Get project name
  static String getProjectName(LogOption option) {
    final String content =
        '${option.language == Language.en ? '   NAME:' : '名称:'} ${option.projectName}';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// Head  and Footer of log
  static String getSupport(LogOption option) {
    final String content = option.language == Language.en
        ? 'SUPPORT: Served by IDKit '
        : '支持: 由 IDKit 提供服务';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// log output split line
  static String getSplitLine(LogOption option) {
    final String content =
        option.splitDecorator! * (option.language == Language.en ? 66 : 60);
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// Log output flag.
  static String getMark(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? '   MARK: $object' : '标识: $object';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// Log output widget.
  static String getWidget(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? ' WIDGET: $object' : '组件: $object';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// Log output method.
  static String getMethod(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? ' METHOD: $object' : '方法: $object';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// log output path.
  static String getPath(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? '   PATH: $object' : '路径: $object';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// number of log output lines.
  static String getCount(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? '   LINE: $object' : '行数: $object';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// Log output content.
  static String getContent(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? 'CONTENT: $object' : '内容: $object';
    return CLog.coloringMethod(option.colorTheme!, content);
  }

  /// Untitled output.
  static String getOutPut(LogOption option, Object object) =>
      CLog.coloringMethod(option.colorTheme!,
          '${option.language == Language.en ? ' ' * 9 : ' ' * 5}$object');

  /// Log output mode judgment method.
  static void outPutModeMethod(
    LogOption option,
    StackTrace stackTrace,
    LogMethod method, {
    String? mark,
    Object? value,
    List<dynamic>? list,
    Iterable<dynamic>? iterable,
    Map<dynamic, dynamic>? map,
    bool isExpand = false,
    String? json,
  }) {
    log(TLog.getHeaderAndFooter(option));
    switch (option.mode) {
      case LogMode.brief:
        log(TLog.getPath(option, ILog.getFilePath(stackTrace)));
        log(TLog.getCount(option, ILog.getLine(stackTrace)));
        switch (method) {
          case LogMethod.list:
            if (isExpand) {
              log(
                TLog.getContent(
                    option,
                    option.language == Language.en
                        ? 'Collection expansion output, Style:(index -> value) ↓↓'
                        : '集合展开输出,样式:(索引 -> 元素) ↓↓'),
              );
              (list ?? []).iFor((index, element) =>
                  log(TLog.getOutPut(option, '$index -> $element')));
            } else {
              log(TLog.getContent(option, list ?? ''));
            }
            break;
          case LogMethod.iterable:
            if (isExpand) {
              log(TLog.getContent(
                option,
                option.language == Language.en
                    ? 'A collection of values or elements expands the output, Style:(index -> value) ↓↓'
                    : '值或者元素的集合展开输出,样式:(索引 -> 元素) ↓↓',
              ));
              iterable!.iFor((index, element) =>
                  log(TLog.getOutPut(option, '$index -> $element')));
            } else {
              log(TLog.getContent(option, iterable ?? ''));
            }
            break;
          case LogMethod.map:
            if (isExpand) {
              log(TLog.getContent(
                option,
                option.language == Language.en
                    ? 'Key-value pair collection expansion output, Style:(key -> value) ↓↓'
                    : '键值对集合展开输出,样式:(键 -> 值) ↓↓',
              ));
              final Iterable<dynamic> keys = (map ?? {}).keys;
              final List<dynamic> keysList = keys.toList();
              keysList.sort((a, b) => '$a'.length.compareTo('$b'.length));
              final int length = '${keysList.last}'.length;
              keys.toList().iFor((_, element) {
                final int curLength = '$element'.length;
                final dynamic value = (map ?? {})[element];
                log(TLog.getOutPut(
                    option, '${' ' * (length - curLength)}$element -> $value'));
              });
            } else {
              log(TLog.getContent(option, map ?? ''));
            }
            break;
          case LogMethod.json:
            log(TLog.getContent(option, json ?? ''));
            break;
          default:
            log(TLog.getContent(option, value ?? ''));
        }
        break;
      case LogMode.full:
        log(TLog.getTime(option));
        log(TLog.getProjectName(option));
        log(TLog.getPath(option, ILog.getFilePath(stackTrace)));
        log(TLog.getWidget(option, ILog.getWidgetName(stackTrace)));
        log(TLog.getMethod(option, ILog.getCallMethod(stackTrace)));
        log(TLog.getCount(option, ILog.getLine(stackTrace)));
        log(TLog.getSupport(option));
        log(TLog.getSplitLine(option));
        switch (method) {
          case LogMethod.list:
            if (isExpand) {
              log(
                TLog.getContent(
                    option,
                    option.language == Language.en
                        ? 'Collection expansion output, Style:(index -> value) ↓↓'
                        : '集合展开输出,样式:(索引 -> 元素) ↓↓'),
              );
              (list ?? []).iFor((index, element) =>
                  log(TLog.getOutPut(option, '$index -> $element')));
            } else {
              log(TLog.getContent(option, list ?? ''));
            }
            break;
          case LogMethod.iterable:
            if (isExpand) {
              log(TLog.getContent(
                option,
                option.language == Language.en
                    ? 'A collection of values or elements expands the output, Style:(index -> value) ↓↓'
                    : '值或者元素的集合展开输出,样式:(索引 -> 元素) ↓↓',
              ));
              iterable!.iFor((index, element) =>
                  log(TLog.getOutPut(option, '$index -> $element')));
            } else {
              log(TLog.getContent(option, iterable ?? ''));
            }
            break;
          case LogMethod.map:
            if (isExpand) {
              log(TLog.getContent(
                option,
                option.language == Language.en
                    ? 'Key-value pair collection expansion output, Style:(key -> value) ↓↓'
                    : '键值对集合展开输出,样式:(键 -> 值) ↓↓',
              ));
              final Iterable<dynamic> keys = (map ?? {}).keys;
              final List<dynamic> keysList = keys.toList();
              keysList.sort((a, b) => '$a'.length.compareTo('$b'.length));
              final int length = '${keysList.last}'.length;
              keys.toList().iFor((_, element) {
                final int curLength = '$element'.length;
                final dynamic value = (map ?? {})[element];
                log(TLog.getOutPut(
                    option, '${' ' * (length - curLength)}$element -> $value'));
              });
            } else {
              log(TLog.getContent(option, map ?? ''));
            }
            break;
          case LogMethod.json:
            log(TLog.getContent(option, json ?? ''));
            break;
          default:
            log(TLog.getContent(option, value ?? ''));
        }

        break;
      default:
        log(TLog.getPath(option, ILog.getFilePath(stackTrace)));
        log(TLog.getWidget(option, ILog.getWidgetName(stackTrace)));
        log(TLog.getMethod(option, ILog.getCallMethod(stackTrace)));
        log(TLog.getCount(option, ILog.getLine(stackTrace)));
        log(TLog.getSplitLine(option));
        switch (method) {
          case LogMethod.list:
            if (isExpand) {
              log(
                TLog.getContent(
                    option,
                    option.language == Language.en
                        ? 'Collection expansion output, Style:(index -> value) ↓↓'
                        : '集合展开输出,样式:(索引 -> 元素) ↓↓'),
              );
              (list ?? []).iFor((index, element) =>
                  log(TLog.getOutPut(option, '$index -> $element')));
            } else {
              log(TLog.getContent(option, list ?? ''));
            }
            break;
          case LogMethod.iterable:
            if (isExpand) {
              log(TLog.getContent(
                option,
                option.language == Language.en
                    ? 'A collection of values or elements expands the output, Style:(index -> value) ↓↓'
                    : '值或者元素的集合展开输出,样式:(索引 -> 元素) ↓↓',
              ));
              iterable!.iFor((index, element) =>
                  log(TLog.getOutPut(option, '$index -> $element')));
            } else {
              log(TLog.getContent(option, iterable ?? ''));
            }
            break;
          case LogMethod.map:
            if (isExpand) {
              log(TLog.getContent(
                option,
                option.language == Language.en
                    ? 'Key-value pair collection expansion output, Style:(key -> value) ↓↓'
                    : '键值对集合展开输出,样式:(键 -> 值) ↓↓',
              ));
              final Iterable<dynamic> keys = (map ?? {}).keys;
              final List<dynamic> keysList = keys.toList();
              keysList.sort((a, b) => '$a'.length.compareTo('$b'.length));
              final int length = '${keysList.last}'.length;
              keys.toList().iFor((_, element) {
                final int curLength = '$element'.length;
                final dynamic value = (map ?? {})[element];
                log(TLog.getOutPut(
                    option, '${' ' * (length - curLength)}$element -> $value'));
              });
            } else {
              log(TLog.getContent(option, map ?? ''));
            }
            break;
          case LogMethod.json:
            log(TLog.getContent(option, json ?? ''));
            break;
          default:
            log(TLog.getContent(option, value ?? ''));
        }
    }
    log(TLog.getHeaderAndFooter(option, type: false));
  }
}

/// Log output method.
enum LogMethod {
  value,
  list,
  iterable,
  map,
  json,
}

mixin CLog {
  /// Coloring
  static String coloringMethod(ColorTheme theme, Object object) {
    int color = 0;
    switch (theme) {
      case ColorTheme.red:
        color = 31;
        break;
      case ColorTheme.yellow:
        color = 33;
        break;
      case ColorTheme.green:
        color = 32;
        break;
      case ColorTheme.blue:
        color = 34;
        break;
      case ColorTheme.cyan:
        color = 36;
        break;
      case ColorTheme.white:
        color = 37;
        break;
      case ColorTheme.black:
        color = 30;
        break;
      default:
        color = 97;
    }
    return '\x1B[${color}m$object\x1B[0m';
  }
}
