import 'dart:async';
import 'dart:math';
import 'package:idkit/idkit.dart';
import 'package:flutter/material.dart';
import 'package:idkit/src/widget/slider/widget/slider_floatingmark.dart';
import 'package:idkit/src/widget/slider/widget/slider_thumb.dart';
import 'package:idkit/src/widget/slider/widget/slider_track.dart';

class CustomBilateralSilder extends StatefulWidget {
  const CustomBilateralSilder({
    super.key,
    this.initValue = 0,
    this.maxValue = 1,
    this.minValue = -1,
    this.thumbSize = 6,
    this.thumbColor = Colors.blue,
    this.thumbAnimationDuration = const Duration(milliseconds: 200),
    this.trackIndent = 3,
    this.trackHeight = 3,
    this.trackColor = Colors.grey,
    this.trackActiveColor = Colors.blue,
    this.trackActiveGradientColors,
    this.trackActiveGradientStops,
    this.trackActiveGradientBegin = Alignment.centerLeft,
    this.trackActiveGradientEnd = Alignment.centerRight,
    this.trackActiveGradientTileMode = TileMode.clamp,
    this.floatingMark = true,
    this.floatingMarkWidth = 1,
    this.floatingMarkIndent = 3,
    this.floatingMarkColor = Colors.blue,
    this.sliderValueMethod,
  })  : assert(initValue <= 1 && initValue >= -1,
            'The value range of initValue of the sliding track is a floating point number from -1.0 to 1.0 .'),
        assert((maxValue <= 1) && maxValue >= minValue,
            'The maxValue of the sliding track is in the range of -1.0 to 1.0 .'),
        assert((minValue >= -1) && minValue <= maxValue,
            'The minValue of the sliding track is in the range of -1.0 to 1.0 .'),
        assert(initValue >= minValue && initValue < maxValue,
            'The initial value of the sliding track must be between the maximum value and the minimum value.');

  /// The initial value of the sliding track.
  /// The initial value range of [initValue] is -1.0 to 1.0.
  final double initValue;

  /// The [maxValue] that the sliding rail can slide.
  /// The [maxValue] ranges from -1.0 to 1.0, and it is greater than or equal to the [minValue].
  final double maxValue;

  /// The [minValue] that the sliding track can slide.
  /// The [minValue] ranges from -1 to 1, and it is less than or equal to the [maxValue].
  final double minValue;

  /// The size of the thumb slider.
  final double thumbSize;

  /// The color of the thumb block.
  final Color thumbColor;

  /// Thumb block animation time.
  final Duration thumbAnimationDuration;

  /// Indentation of sliding track.
  final double trackIndent;

  /// Height of sliding track.
  final double trackHeight;

  /// The color of the slide track.
  final Color trackColor;

  /// The active color of the slider track.
  final Color trackActiveColor;

  /// A collection of active gradient colors for the slide track.
  final List<Color>? trackActiveGradientColors;

  /// Active track gradient color ratio.
  final List<double>? trackActiveGradientStops;

  /// Active track fade start orientation.
  final AlignmentGeometry trackActiveGradientBegin;

  /// Active track fade end orientation.
  final AlignmentGeometry trackActiveGradientEnd;

  /// Active track gradient tile mode.
  final TileMode trackActiveGradientTileMode;

  /// Whether to enable the auxiliary hover function. It only works when using Apple Pecile.
  final bool floatingMark;

  /// The width of the floating mark.
  final double floatingMarkWidth;

  /// Indentation of floating mark.
  final double floatingMarkIndent;

  /// The color of the floating mark.
  final Color floatingMarkColor;

  /// Callback function for the current sliding value of the sliding track.
  final Function(double value)? sliderValueMethod;

  @override
  State<CustomBilateralSilder> createState() => _CustomBilateralSilderState();
}

class _CustomBilateralSilderState extends State<CustomBilateralSilder>
    with SingleTickerProviderStateMixin {
  /// Swipe track subscribers.
  final StreamController<double> sliderStreamController =
      StreamController.broadcast();

  ///  An animation controller for the thumb block.
  late AnimationController thumbAnimationController;

  /// Subscribers to the floating mark.
  late StreamController<FloatingMarkInfo> floatingMarkStream =
      StreamController<FloatingMarkInfo>.broadcast();

  /// The display mode of the active track.
  late bool isActiveTrackMode = false;

  /// Width of the slide track part.
  late double sliderWWidth = 0;

  /// Slide track initial offset position.
  late double initLocation = 0;

  /// Maximum offset position of sliding track.
  late double maxOffset = 0;

  /// Minimum offset position of sliding track.
  late double minOffset = 0;

  /// Is it possible to operate.
  late bool isManipulate = false;

  /// Calculation of the position of the sliding track.
  void _getInitialOffset() {
    final double oPercentage =
        widget.initValue.range(min: widget.minValue, max: widget.maxValue);
    final double location = _getOffsetPosition(oPercentage);
    sliderStreamController.add(location);
  }

  /// Get the sliding maximum and minimum values of the sliding rail component.
  double _getOffsetPosition(double value) {
    double location = 0;
    if (value <= 0) {
      location = widget.trackIndent +
          (sliderWWidth - widget.trackIndent * 2) * 0.5 * (1 + value);
    } else {
      location = sliderWWidth * 0.5 +
          (sliderWWidth - widget.trackIndent * 2) * 0.5 * value;
    }
    return location;
  }

  /// Get the current sliding value of the sliding track.
  double _getSliderValue(double offset) {
    final double middleLength = (sliderWWidth - widget.trackIndent * 2) * 0.5;
    return (offset - sliderWWidth * 0.5) / middleLength;
  }

  @override
  void initState() {
    thumbAnimationController = AnimationController(
      vsync: this,
      duration: widget.thumbAnimationDuration,
      reverseDuration: widget.thumbAnimationDuration,
      lowerBound: 1,
      upperBound: 1.5,
    );
    isActiveTrackMode = widget.trackActiveGradientColors == null ||
        widget.trackActiveGradientColors!.length < 2;
    WidgetsBinding.instance.addPostFrameCallback((_) {
      maxOffset = _getOffsetPosition(widget.maxValue);
      minOffset = _getOffsetPosition(widget.minValue);
      _getInitialOffset();
    });
    super.initState();
  }

  @override
  void didUpdateWidget(covariant CustomBilateralSilder oldWidget) {
    if (oldWidget.trackActiveGradientColors !=
        widget.trackActiveGradientColors) {
      isActiveTrackMode = widget.trackActiveGradientColors == null ||
          widget.trackActiveGradientColors!.length < 2;
    }
    if (oldWidget.initValue != widget.initValue) _getInitialOffset();
    if (oldWidget.minValue != widget.minValue) {
      minOffset = _getOffsetPosition(widget.minValue);
      initLocation = _getOffsetPosition(
          widget.initValue.range(min: widget.minValue, max: widget.maxValue));
      sliderStreamController.add(initLocation);
    }
    if (oldWidget.maxValue != widget.maxValue) {
      maxOffset = _getOffsetPosition(widget.maxValue);
      initLocation = _getOffsetPosition(
          widget.initValue.range(min: widget.minValue, max: widget.maxValue));
      sliderStreamController.add(initLocation);
    }
    if (oldWidget.trackIndent != widget.trackIndent) {
      minOffset = _getOffsetPosition(widget.minValue);
      maxOffset = _getOffsetPosition(widget.maxValue);
      initLocation = _getOffsetPosition(
          widget.initValue.range(min: widget.minValue, max: widget.maxValue));
      sliderStreamController.add(initLocation);
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints boxConstraints) {
        sliderWWidth = boxConstraints.biggest.width;
        return MouseRegion(
          onHover: widget.floatingMark
              ? (event) {
                  if (!isManipulate) {
                    floatingMarkStream.add(
                      FloatingMarkInfo(
                        state: true,
                        position: event.localPosition.dx
                            .range(min: minOffset, max: maxOffset),
                      ),
                    );
                  }
                }
              : null,
          onEnter: widget.floatingMark
              ? (event) {
                  if (!isManipulate) {
                    floatingMarkStream.add(
                      FloatingMarkInfo(
                        state: true,
                        position: event.localPosition.dx
                            .range(min: minOffset, max: maxOffset),
                      ),
                    );
                  }
                }
              : null,
          onExit: widget.floatingMark
              ? (event) {
                  if (!isManipulate) {
                    floatingMarkStream.add(
                      FloatingMarkInfo(
                        state: false,
                        position: event.localPosition.dx
                            .range(min: minOffset, max: maxOffset),
                      ),
                    );
                  }
                }
              : null,
          child: GestureDetector(
            behavior: HitTestBehavior.translucent,
            onHorizontalDragDown: (details) {
              thumbAnimationController.forward();
              if (widget.floatingMark) {
                isManipulate = true;
                floatingMarkStream.add(const FloatingMarkInfo(state: false));
              }
            },
            onHorizontalDragUpdate: (details) {
              final double sliderLength = details.localPosition.dx
                  .range(min: minOffset, max: maxOffset);
              sliderStreamController.add(sliderLength);
              widget.sliderValueMethod?.call(_getSliderValue(sliderLength));
            },
            onHorizontalDragEnd: (details) {
              thumbAnimationController.reverse();
              if (widget.floatingMark) {
                isManipulate = false;
              }
            },
            onHorizontalDragCancel: () {
              if (widget.floatingMark) {
                isManipulate = false;
              }
            },
            child: Stack(
              children: [
                Positioned.fill(
                  child: Align(
                    child: SliderTrack(
                      key: UniqueKey(),
                      height: widget.trackHeight,
                      indent: widget.trackIndent,
                      color: widget.trackColor,
                    ),
                  ),
                ),
                Positioned.fill(
                  left: sliderWWidth * 0.5,
                  child: StreamBuilder<double>(
                    initialData: initLocation,
                    stream: sliderStreamController.stream,
                    builder: (context, snapshot) {
                      final double length = snapshot.data! - sliderWWidth * 0.5;
                      return Align(
                        alignment: Alignment.centerLeft,
                        child: Transform.rotate(
                          alignment: Alignment.centerLeft,
                          angle: length.isNegative ? -pi : 0,
                          child: Container(
                            key: UniqueKey(),
                            width: length.abs(),
                            height: widget.trackHeight,
                            decoration: BoxDecoration(
                              color: widget.trackActiveColor,
                              gradient: isActiveTrackMode
                                  ? null
                                  : LinearGradient(
                                      begin: widget.trackActiveGradientBegin,
                                      end: widget.trackActiveGradientEnd,
                                      colors: widget.trackActiveGradientColors!,
                                      stops: widget.trackActiveGradientStops,
                                      tileMode:
                                          widget.trackActiveGradientTileMode,
                                    ),
                            ),
                          ),
                        ),
                      );
                    },
                  ),
                ),
                StreamBuilder<FloatingMarkInfo>(
                  initialData: const FloatingMarkInfo(),
                  stream: floatingMarkStream.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data!.position -
                          widget.floatingMarkWidth * 0.5,
                      child: Visibility(
                        visible: snapshot.data!.state,
                        child: Align(
                          alignment: Alignment.centerLeft,
                          child: FloatingMark(
                            width: widget.floatingMarkWidth,
                            color: widget.floatingMarkColor,
                            indent: widget.floatingMarkIndent,
                          ),
                        ),
                      ),
                    );
                  },
                ),
                StreamBuilder<double>(
                  initialData: initLocation,
                  stream: sliderStreamController.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data! - widget.thumbSize * 0.5,
                      child: Align(
                        alignment: Alignment.centerLeft,
                        child: AnimatedBuilder(
                          animation: thumbAnimationController,
                          builder: (context, child) => Transform.scale(
                            scale: thumbAnimationController.value,
                            child: child,
                          ),
                          child: SliderThumb(
                            size: widget.thumbSize,
                            color: widget.thumbColor,
                          ),
                        ),
                      ),
                    );
                  },
                ),
              ],
            ),
          ),
        );
      },
    );
  }

  @override
  void dispose() {
    sliderStreamController.close();
    floatingMarkStream.close();
    thumbAnimationController.dispose();
    super.dispose();
  }
}
