import 'dart:ui';
import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/cupertino.dart' hide Image;
import 'package:flutter/services.dart';
import 'package:idkit/idkit.dart';

class IDKitImage extends StatelessWidget {
  const IDKitImage({
    super.key,
    this.image,
    this.byteData,
    this.byteBuffer,
    this.immutableBuffer,
    this.uint8List,
    this.uint32List,
    this.pixelFormat,
    this.width,
    this.height,
    this.activityIndicatorColor,
    this.loadingChild,
    this.errorChild,
  });

  /// Image resource loading method of Image type.
  const IDKitImage.image({
    super.key,
    required this.image,
    this.width,
    this.height,
    this.activityIndicatorColor,
    this.loadingChild,
    this.errorChild,
  })  : byteData = null,
        byteBuffer = null,
        immutableBuffer = null,
        uint8List = null,
        uint32List = null,
        pixelFormat = null;

  /// Image resource loading method of ByteData type.
  const IDKitImage.byteData({
    super.key,
    required this.byteData,
    this.width,
    this.height,
    this.activityIndicatorColor,
    this.loadingChild,
    this.errorChild,
  })  : image = null,
        byteBuffer = null,
        immutableBuffer = null,
        uint8List = null,
        uint32List = null,
        pixelFormat = null;

  /// Image resource loading method of ByteBuffer type.
  const IDKitImage.byteBuffer({
    super.key,
    required this.byteBuffer,
    this.width,
    this.height,
    this.activityIndicatorColor,
    this.loadingChild,
    this.errorChild,
  })  : image = null,
        byteData = null,
        immutableBuffer = null,
        uint8List = null,
        uint32List = null,
        pixelFormat = null;

  /// Image resource loading method of ImmutableBuffer type.
  const IDKitImage.immutableBuffer({
    super.key,
    required this.immutableBuffer,
    this.width,
    this.height,
    this.activityIndicatorColor,
    this.loadingChild,
    this.errorChild,
  })  : image = null,
        byteData = null,
        byteBuffer = null,
        uint8List = null,
        uint32List = null,
        pixelFormat = null;

  /// Image resource loading method of Uint8List type.
  const IDKitImage.uint8List({
    super.key,
    required this.uint8List,
    this.width,
    this.height,
    this.activityIndicatorColor,
    this.loadingChild,
    this.errorChild,
  })  : image = null,
        byteData = null,
        byteBuffer = null,
        immutableBuffer = null,
        uint32List = null,
        pixelFormat = null;

  /// Image resource loading method of Uint32List type.
  const IDKitImage.uint32List({
    super.key,
    required this.uint32List,
    required double iWidth,
    required double iHeight,
    this.activityIndicatorColor,
    this.pixelFormat = PixelFormat.rgba8888,
    this.loadingChild,
    this.errorChild,
  })  : width = iWidth,
        height = iHeight,
        image = null,
        byteData = null,
        byteBuffer = null,
        immutableBuffer = null,
        uint8List = null;

  /// Image resources of type ui.Image.
  final Image? image;

  /// Image resource of type byteData.
  final ByteData? byteData;

  /// Image resource of type byteBuffer.
  final ByteBuffer? byteBuffer;

  /// Image resource of type immutableBuffer.
  final ImmutableBuffer? immutableBuffer;

  /// Image resource of type uint8List.
  final Uint8List? uint8List;

  /// Image resource of type uint32List.
  final Uint32List? uint32List;

  /// The format of pixel data given to [decodeImageFromPixels].
  final PixelFormat? pixelFormat;

  /// Set the width of the displayed image.
  final double? width;

  /// Set the height of the displayed image.
  final double? height;

  /// Widget displayed on image load.
  final Widget? loadingChild;

  /// Widget for image load failure display.
  final Widget? errorChild;

  /// ActivityIndicator Color
  final Color? activityIndicatorColor;

  /// Get the loaded image resource object.
  Future<Image> _getImage({
    Image? image,
    ByteData? byteData,
    ByteBuffer? byteBuffer,
    ImmutableBuffer? immutableBuffer,
    Uint8List? uint8List,
    Uint32List? uint32List,
  }) async {
    Completer<Image> completer = Completer();
    if (image != null) {
      completer.complete(image);
    } else if (byteData != null) {
      final Codec codec = await instantiateImageCodec(
        byteData.buffer.asUint8List(),
        targetWidth: width?.toInt(),
        targetHeight: height?.toInt(),
      );
      final FrameInfo frameInfo = await codec.getNextFrame();
      completer.complete(frameInfo.image);
    } else if (byteBuffer != null) {
      final Codec codec = await instantiateImageCodec(
        byteBuffer.asUint8List(),
        targetWidth: width?.toInt(),
        targetHeight: height?.toInt(),
      );
      final FrameInfo frameInfo = await codec.getNextFrame();
      completer.complete(frameInfo.image);
    } else if (immutableBuffer != null) {
      final Codec codec = await instantiateImageCodecFromBuffer(
        immutableBuffer,
        targetWidth: width?.toInt(),
        targetHeight: height?.toInt(),
      );
      final FrameInfo frameInfo = await codec.getNextFrame();
      completer.complete(frameInfo.image);
    } else if (uint8List != null) {
      final Codec codec = await instantiateImageCodec(
        uint8List,
        targetWidth: width?.toInt(),
        targetHeight: height?.toInt(),
      );
      final FrameInfo frameInfo = await codec.getNextFrame();
      completer.complete(frameInfo.image);
    } else if (uint32List != null) {
      final Uint8List uint8list = Uint8List.view(uint32List.buffer);
      decodeImageFromPixels(
        uint8list,
        width!.toInt(),
        height!.toInt(),
        pixelFormat!,
        (image) => completer.complete(image),
      );
    } else {
      completer.completeError('Image loading error!');
    }
    return completer.future;
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (_, BoxConstraints boxConstraints) {
        return FutureBuilder<Image>(
          future: _getImage(
            image: image,
            byteData: byteData,
            byteBuffer: byteBuffer,
            immutableBuffer: immutableBuffer,
            uint8List: uint8List,
            uint32List: uint32List,
          ),
          builder: (context, snapshot) {
            if (snapshot.connectionState == ConnectionState.waiting ||
                snapshot.connectionState == ConnectionState.active) {
              return SizedBox(
                width: width ?? 200,
                height: height ?? 200,
                child: Center(
                  child: loadingChild ??
                      CupertinoActivityIndicator(
                        radius: 61.8,
                        color: activityIndicatorColor,
                      ),
                ),
              );
            }
            if (snapshot.connectionState == ConnectionState.done &&
                snapshot.hasError) {
              return SizedBox(
                width: width ?? 200,
                height: height ?? 200,
                child: errorChild ??
                    Center(
                      child: const Text('Image loading failed!')
                          .insetsSymmetric(vertical: 5, horizontal: 5),
                    ),
              );
            }
            final Image loadImg = snapshot.data!;
            final double imgW = width ?? loadImg.width * 1;
            final double imgH = height ?? loadImg.height * 1;
            return CustomPaint(
              painter: ImagePainter(loadImg),
              size: Size(imgW, imgH),
            );
          },
        );
      },
    );
  }
}

/// image loading
class ImagePainter extends CustomPainter {
  const ImagePainter(this.image);
  final Image image;

  @override
  void paint(Canvas canvas, Size size) {
    canvas.drawImageRect(
      image,
      Offset.zero & Size(image.width * 1.0, image.height * 1.0),
      Offset.zero & size,
      Paint(),
    );
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) => true;
}
